@@ -3,12 +3,14 @@
 '''
 
 import os
+import sys
 import uuid
 import json
 import argparse
 import base64
 import binascii
 import datetime
+from collections import namedtuple
 from urllib.parse import urlparse
 
 from tornado.httpserver import HTTPServer
@@ -165,7 +167,10 @@ class MyHelpers:
                 except FileExistsError:
                     pass
             if f is None:
-                raise self.json_error(500, 'Internal server error')
+                msg = 'Internal server error'
+                if self.application.debug:
+                    msg += ' (filename conflict)'
+                raise self.json_error(500, msg)
             f.write(content)
             f.close()
             file_obj.actual_name = actual_name
@@ -907,10 +912,6 @@ class MyApplication(Application):
         admin=(),
         autoreload=True,
     ):
-
-        if prefix.startswith('/healthz/'):
-            raise ValueError("API prefix may not start with /healthz/")
-
         handlers = [
             (prefix, HomePage),
             (prefix + r'(favicon\.ico)', Static),
@@ -968,7 +969,7 @@ class MockAuth(HubAuthenticated):
         return {'name': user}
 
 
-def main():  # pragma: no cover
+def main(argv=None):  # pragma: no cover
     'Main function'
     parser = argparse.ArgumentParser(
         description='ngshare, a REST API nbgrader exchange'
@@ -1008,15 +1009,23 @@ def main():  # pragma: no cover
         action='store_true',
         help='do not automatically upgrade database',
     )
-    args = parser.parse_args()
+    args = parser.parse_args(argv)
     if args.jupyterhub_api_url is not None:
         os.environ['JUPYTERHUB_API_URL'] = args.jupyterhub_api_url
 
-    if not args.no_upgrade_db:
-        dbutil.upgrade(args.database)
-
     prefix = args.prefix or os.environ.get('JUPYTERHUB_SERVICE_PREFIX', '/api/')
 
+    if not prefix.startswith('/') or not prefix.endswith('/'):
+        raise ValueError('API prefix should start and end with /')
+
+    if prefix.startswith('/healthz/'):
+        raise ValueError("API prefix may not start with /healthz/")
+
+    if not args.no_upgrade_db:
+        CmdOpts = namedtuple('CmdOpts', ['x'])
+        cmd_opts = CmdOpts(['data=true', 'storage=' + args.storage])
+        dbutil.upgrade(args.database, cmd_opts=cmd_opts)
+
     if args.vngshare:
         MyRequestHandler.__bases__ = (MockAuth, RequestHandler, MyHelpers)
 
@@ -1050,4 +1059,4 @@ def main():  # pragma: no cover
 
 
 if __name__ == '__main__':  # pragma: no cover
-    main()
+    main(sys.argv[1:])
